/* Copyright (c) 2009, Delft University of Technology

    This file is part of the MEGS package.

    The MEGS pakcage is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The MEGS pakcage is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the MEGS package.  If not, see
    <http://www.gnu.org/licenses/>.
*/
package engine;
import java.io.*; 
import java.util.StringTokenizer;

/**
 * @author menno
 * Reads the data from a single file, all events are assumed to be ordered
 * Takes usr3 output
 */
public class SingleUsr3FileReader implements EventReader {

	private BufferedReader reader;
	
	
	/**
	 * The String representing the next event, this is needed to implement hasMoreEvents()
	 */
	private String nextString;

	/**
	 * Constructs a reader from file f
	 * @param f
	 */
	public SingleUsr3FileReader(File f) throws FileNotFoundException
	{
		reader = new BufferedReader(new FileReader(f));
		loadEvent();
		
	}
	
	/* (non-Javadoc)
	 * @see EventReader#hasMoreEvents()
	 */
	public boolean hasMoreEvents() {
		return (nextString != null);
	}

	/* (non-Javadoc)
	 * @see EventReader#nextEvent()
	 */
	public Event nextEvent() {
		if(nextString == null)
		{
			return null;
		}
		
		Event event = createEvent(nextString);
		
		loadEvent();
		
		return event;
	}
	
	/**
	 * Puts the next event into nextString
	 */
	private void loadEvent()
	{
		try
		{
			nextString = reader.readLine();
		}catch(IOException e)
		{
			System.out.println("Error reading next event: " + e);
		}
	}
	
	/**
	 * Takes a string and creates an event. String format is:
	 * String: Nodenumber - hours:minutes:seconds: Message
	 * Message is Assert : <number> or Status : name : value
	 * Seconds is a double
	 * @param s
	 * @return
	 */
	public static Event createEvent(String s)
	{
		//Ignore the sims
		if(s.startsWith("SIM"))
		{
			return null;
		}
		
		StringTokenizer tok = new StringTokenizer(s, " :");
		
		try
		{
			//parse the intro if not ignore the event		

			int node = Integer.parseInt(tok.nextToken());
			//skip the -
			tok.nextToken();
			int hours = Integer.parseInt(tok.nextToken());
			int minutes = Integer.parseInt(tok.nextToken());
			double seconds = Double.parseDouble(tok.nextToken());
			double time = 60*60*hours + 60*minutes + seconds;
			
			String assertOrStatus = tok.nextToken();
			if(assertOrStatus.equals("Assert"))
			{
				AssertEvent e = new AssertEvent();
				//make an assert
				e.node = node;
				e.time = time;
				e.assertNum = Integer.parseInt(tok.nextToken());
				
		//		String assertString = "";
		//		while(tok.hasMoreTokens())
		//		{
		//			assertString += tok.nextToken() + " " ;
		//		}
			//	e.assertString = assertString.trim();
				
				return e;
			}
			else if(assertOrStatus.equals("Status"))
			{
				//make a status update
				StatusEvent e = new StatusEvent();
				
				e.node = node;
				e.time = time;
				e.name = tok.nextToken();
				e.value = Integer.parseInt(tok.nextToken());
				return e;
			}
			else
			{
				System.out.println("Unrecognized event: " + s);
				return null;
			}

		}catch(Exception e)
		{
			System.out.println("Error creating event " + e + " on String: " + s);
		}
		return null;
	}

}
