/* Copyright (c) 2009, Delft University of Technology

    This file is part of the MEGS package.

    The MEGS pakcage is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The MEGS pakcage is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the MEGS package.  If not, see
    <http://www.gnu.org/licenses/>.
*/
package mintroute;

import java.io.*;
import engine.*;


public class ExternalMain {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws FileNotFoundException{
		
		File file = null;
		File[] files = null;
		boolean printusage = false;
		boolean uart = false;
		boolean uartsplit = false;
		boolean usr3 = false;
		if(args.length == 2)
		{
			
			if(args[1].equals("uart"))
			{
				uart = true;
			}
			else if(args[1].equals("uartsplit"))
			{
				uartsplit = true;
			}
			else if(args[1].equals("usr3"))
			{
				usr3 = true;
			}
			else
			{
				printusage= true;
			}
			
			//single file
			if(uart || usr3)
			{
				file = new File(args[0]);
				if(!file.exists())
				{
					System.out.println("File doesn't exist: " + file);
					System.exit(1);
				}
			}
			//multiple files
			if(uartsplit)
			{
				
				String filename = args[0];
				System.out.println("Checking for files: " + filename);
				if(filename.contains("^^"))
				{
					files = MultipleUARTFileReader.makeFileList(filename);
				
				}
				else
				{
					printusage = true;
				}
			}
		}
		else
		{
			printusage= true;
		}
		
		if(printusage)
		{
			System.out.println("Usage: java engine.ExternalMain <filename> uart|usr3");
			System.out.println("Multiple files: java engine.ExternalMain <filename> uartsplit (^^ must be in filename and is replaced by 00 to 99)");
			System.exit(2);
		}
		
		
		EventReader reader;
		
		if(uart && file != null)
		{
			reader = new SingleUARTFileReader(file);
		}
		else if(uartsplit && files != null)
		{
			reader = new MultipleUARTFileReader(files);
		}
		else if(usr3 && file != null)
		{
			reader = new SingleUsr3FileReader(file);
		}
		else
		{
			//one of the options must have been selected or the message should have printed
			assert false : "No method choosen";
			// shouldn't be reached
			reader = null;
		}
		

		SystemStatus system = new MintRouteSystem();
		Engine eng = new Engine(reader, system);
		
		eng.run();
		

	}

}
